@@ -164,14 +164,15 @@ things0/
 
 
 @mock_s3
-def test_TupleS3StoreBackend():
+def test_TupleS3StoreBackend_with_prefix():
     """
     What does this test test and why?
 
     We will exercise the store backend's set method twice and then verify
     that the we calling get and list methods will return the expected keys.
 
-    We will also check that the objects are stored on S3 at the expected location.
+    We will also check that the objects are stored on S3 at the expected location,
+    and that the correct S3 URL for the object can be retrieved.
 
     """
     bucket = "leakybucket"
@@ -206,6 +207,57 @@ def test_TupleS3StoreBackend():
         [s3_object_info['Key'] for s3_object_info in
          boto3.client('s3').list_objects(Bucket=bucket, Prefix=prefix)['Contents']]) == {
         'this_is_a_test_prefix/my_file_AAA', 'this_is_a_test_prefix/my_file_BBB'}
+    
+    assert my_store.get_url_for_key(('AAA',)) == 'https://s3.amazonaws.com/%s/%s/my_file_AAA' % (bucket, prefix)
+    assert my_store.get_url_for_key(('BBB',)) == 'https://s3.amazonaws.com/%s/%s/my_file_BBB' % (bucket, prefix)
+
+
+@mock_s3
+def test_TupleS3StoreBackend_with_empty_prefixes():
+    """
+    What does this test test and why?
+
+    We will exercise the store backend's set method twice and then verify
+    that the we calling get and list methods will return the expected keys.
+
+    We will also check that the objects are stored on S3 at the expected location,
+    and that the correct S3 URL for the object can be retrieved.
+    """
+    bucket = "leakybucket"
+    prefix = ""
+
+    # create a bucket in Moto's mock AWS environment
+    conn = boto3.resource('s3', region_name='us-east-1')
+    conn.create_bucket(Bucket=bucket)
+
+    my_store = TupleS3StoreBackend(
+        filepath_template="my_file_{0}",
+        bucket=bucket,
+        prefix=prefix,
+    )
+
+    # We should be able to list keys, even when empty
+    keys = my_store.list_keys()
+    assert len(keys) == 0
+
+    my_store.set(("AAA",), "aaa", content_type='text/html; charset=utf-8')
+    assert my_store.get(("AAA",)) == 'aaa'
+
+    obj = boto3.client('s3').get_object(Bucket=bucket, Key=prefix + "/my_file_AAA")
+    assert obj["ContentType"] == 'text/html; charset=utf-8'
+    assert obj["ContentEncoding"] == 'utf-8'
+
+    my_store.set(("BBB",), "bbb")
+    assert my_store.get(("BBB",)) == 'bbb'
+
+    assert set(my_store.list_keys()) == {("AAA",), ("BBB",)}
+    assert set(
+        [s3_object_info['Key'] for s3_object_info in
+         boto3.client('s3').list_objects(Bucket=bucket, Prefix=prefix)['Contents']]) == {
+        'my_file_AAA', 'my_file_BBB'}
+
+    assert my_store.get_url_for_key(('AAA',)) == 'https://s3.amazonaws.com/leakybucket/my_file_AAA'
+    assert my_store.get_url_for_key(('BBB',)) == 'https://s3.amazonaws.com/leakybucket/my_file_BBB'
 
 
 def test_TupleGCSStoreBackend():
@@ -215,7 +267,6 @@ def test_TupleGCSStoreBackend():
 
     Since no package like moto exists for GCP services, we mock the GCS client
     and assert that the store backend makes the right calls for set, get, and list.
-
     """
     bucket = "leakybucket"
     prefix = "this_is_a_test_prefix"
